/************************************************************************/
/* bind.c: This file contains routines for DAP binding to DSA. 		*/
/*                                                                      */
/* Modification Log:                                                    */
/*                                                                      */
/* 91/05/21     Sze-Ying Wuu: Additions/changes made for xdi, v 1.0     */
/* 91/01/11     /f/osi/others/quipu/dish/bind.c,v 7.2           	*/
/*              Isode Interim (6.8)                               	*/
/************************************************************************/

/* $Header: bind.c,v 1.8 92/09/01 16:32:54 sywuu Exp $ */

/*
 *    Original source: ISODE Interim Release 6.8
 *    Changes/additions by: Sze-Ying Wuu, Bellcore, <sywuu@thumper.bellcore.com>
 *
 *               Changes and Additions Copyright (c) 1991 Bellcore
 *                            All Rights Reserved
 *
 *    Permission is granted to copy, modify and redistribute this program
 *    in source or binary form provided that it is not sold for
 *    profit, and all copyright notices, permission and nonwarranty notices 
 *    remain intact.
 *
 *    Bellcore makes no warranty and accepts no liability for this program.
 * 
 *
 *                             ISODE NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>
#include "quipu/util.h"
#include "quipu/common.h"
#include "quipu/entry.h"
#include "quipu/dua.h"
#include "quipu/name.h"
#include "quipu/bind.h"
#include "quipu/dsp.h"
#include "quipu/ds_error.h"
#include "tailor.h"
#include "defs.h"
#include "sequence.h"

DN	        user_name;
char		bound;

extern	PS	opt;

extern int	dsap_ad;

extern char	passwd[];
extern char	namestr[];
extern Attr_Sequence oclass;
extern str_seq 	dnseq, backseq, showseq;
extern int	dn_number, back_buf_num;
static int	auth_type = DBA_AUTH_SIMPLE; 

char		neverefer = FALSE;

static struct ds_bind_arg bindarg;
static struct ds_bind_arg bindresult;
static struct ds_bind_error binderr;

int 	main_dsa_id;
static int	referral_dsa;

#ifndef NO_STATS
extern LLog * log_stat;
#endif 
extern LLog * log_dsap;

extern char *new_version();
extern long hash_passwd();
extern char *hash2str();
/*
extern struct signature *sign_operation();
*/
extern int encode_DAS_TokenToSign();


/* bind to DSA */
char *cnnct_bind()
{
  extern  char * dsa_address,
               * myname;
  extern  char * tailfile;
  FILE * fp;
  char buf[BUFSIZ], bindpass[STRINGLEN];

  if (*passwd != 0)
    (void) strcpy(bindpass, "******");
  else
    bindpass[0] = '\0';

  /* set dsa_address */
  dsa_address = NULLCP;
  /* read tailor file to get address */
  if( (fp = fopen(isodefile(tailfile,0), "r")) == (FILE *) NULL) {
    return("Cannot open `dsaptailor' file.\nClick on this window to continue.");
  }
  while(fgets(buf, sizeof(buf), fp) != NULLCP)
    if ( (*buf != '#') && (*buf != '\n') )
      (void) tai_string (buf);

  (void) fclose(fp);

  if (dsa_address == NULLCP)
    dsa_address = myname;

  /* set password */
  if (bindpass[0] != 0) {
    if (strcmp (bindpass,"******") != 0)
      (void) strcpy (passwd,bindpass);
  } else
    passwd[0] = 0;

  /* now bind */
  bindarg.dba_version = DBA_VERSION_V1988;
  if (passwd[0] == 0) {
    bindarg.dba_passwd_len = 0;
    bindarg.dba_passwd [0] = '\0';
  } else {
    bindarg.dba_passwd_len = strlen(passwd);
    (void) strcpy (bindarg.dba_passwd, passwd);
  }
 
  if (bindarg.dba_passwd_len == 0) auth_type = DBA_AUTH_NONE;
  if (*namestr == 0) {
	auth_type = DBA_AUTH_NONE;
	bindarg.dba_dn = NULLDN;
  }
  else 
        bindarg.dba_dn = str2dn(*namestr != '@' ? namestr : namestr+1);

  bindarg.dba_auth_type = auth_type;
  switch (auth_type) {
  case DBA_AUTH_NONE:
  case DBA_AUTH_SIMPLE:
       break;
  case DBA_AUTH_PROTECTED:
       protect_password();
       break;
  case DBA_AUTH_STRONG:
       sign_bindarg();
       break;
  }


  bound = FALSE;
  if (secure_ds_bind (&bindarg, &binderr, &bindresult) != DS_OK) {
    return(binderr.dbe_type == DBE_TYPE_SECURITY?
           "Security error - Check name and password.\nClick on this window to exit.":
           "Service error - Can't connect to directory!\nClick on this window to exit.");
  } else {
    bound = TRUE;
    user_name = bindarg.dba_dn;
    main_dsa_id = dsap_ad;

#ifndef NO_STATS
    LLOG (log_stat,LLOG_NOTICE,("bound ('%s' to '%s')",namestr,dsa_address));
#endif
#ifndef NO_STATS
    LLOG (log_stat,LLOG_NOTICE,("xdi bound to directory"));
#endif

    set_default_type();
    check_known_oids();
    oclass = as_comp_new(AttrT_new("2.5.4.0"), NULLAV, NULLACL_INFO);
  }
  dn_number = 0;
  back_buf_num = 0;
  backseq = dnseq = NULLDS;
  return NULLCP;
}


referral_bind (addr) 
struct PSAPaddr * addr;
{
  if (referral_dsa != 0) 
	(void) dap_unbind (referral_dsa++);
  else
	referral_dsa = dsap_ad + 1;

  dsap_ad = referral_dsa;

  /* prepare credentials */
  bindarg.dba_auth_type = auth_type;
  switch (auth_type) {
  case DBA_AUTH_NONE:
  case DBA_AUTH_SIMPLE:
	break;
  case DBA_AUTH_PROTECTED: 
	protect_password();
	break;
  case DBA_AUTH_STRONG:
	sign_bindarg();
	break;
  }

  binderr.dbe_value = 0;
  if (dap_bind (&dsap_ad, &bindarg, &binderr, &bindresult, addr) != OK) {
	if (binderr.dbe_value == 0)
		ps_print (opt, "*** Service error : Unable to contact DSA ***\n");
	else
		ds_bind_error(opt, &binderr);
	referral_dsa = 0;
	dsap_ad = main_dsa_id;
	return (0);
  }
  referral_dsa = dsap_ad;

#ifndef NO_STATS
  LLOG (log_stat,LLOG_NOTICE,("referral connect"));
#endif

  return (1);
}

static int protect_password()
{
  long hash;
  char *cp;
  int len;

  bindarg.dba_time1 = new_version();
  bindarg.dba_time2 = NULLCP;
  bindarg.dba_r1.n_bits = 0;
  bindarg.dba_r1.value = NULLCP;
  bindarg.dba_r2.n_bits = 0;
  bindarg.dba_r2.value = NULLCP;
  hash = 0;
  hash = hash_passwd(hash, passwd, strlen(passwd));
  hash = hash_passwd(hash, bindarg.dba_time1, strlen(bindarg.dba_time1));
  cp = hash2str(hash, &len);
  bcopy(cp, bindarg.dba_passwd, len);
  bindarg.dba_passwd_len = len;
}

static int sign_bindarg()
{
  bindarg.dba_time1 = new_version();
  /* Have to send a random number, but don't care what it is */
  bindarg.dba_r1.n_bits = 8;
  bindarg.dba_r1.value = strdup("*");

  bindarg.dba_sig =  (struct signature *) NULL;
/*
  bindarg.dba_sig = sign_operation((caddr_t) &bindarg, encode_DAS_TokenToSign);
*/
}

