/************************************************************************/
/* main.c: The main routine of xdi.					*/
/*                                                                      */
/* Modification Log:                                                    */
/*                                                                      */
/* 91/05/21     Sze-Ying Wuu: Additions/changes made for xdi, v 1.0     */
/* 91/02/22     /f/osi/others/quipu/uips/pod/RCS/main.c,v 7.2           */
/*              Isode Interim (6.8)                               	*/
/************************************************************************/

/* $Header: main.c,v 1.8 92/09/01 16:34:53 sywuu Exp $ */

/*
 *    Original source: ISODE Interim Release 6.8
 *    Changes/additions by: Sze-Ying Wuu, Bellcore, <sywuu@thumper.bellcore.com>
 *
 *               Changes and Additions Copyright (c) 1991 Bellcore
 *                            All Rights Reserved
 *
 *    Permission is granted to copy, modify and redistribute this program
 *    in source or binary form provided that it is not sold for
 *    profit, and all copyright notices, permission and nonwarranty notices 
 *    remain intact.
 *
 *    Bellcore makes no warranty and accepts no liability for this program.
 * 
 *
 *                             ISODE NOTICE
 *
 *    Acquisition, use, and distribution of this module and related
 *    materials are subject to the restrictions of a license agreement.
 *    Consult the Preface in the User's Manual for the full terms of
 *    this agreement.
 *
 */


#include "quipu/util.h"
#include "quipu/photo.h"
#include "quipu/common.h"
#include "quipu/entry.h"
#include "quipu/dua.h"
#include "xdi.h"
#include "select.h"

#include <varargs.h>

extern int print_parse_errors;
extern char *oidtable, *tailfile, *myname, *local_dit;
extern char namestr[], passwd[], base_path[], friendly_base_path[];
extern Widget toplevel;
extern bool testing;
extern ToggleList dua_tlist;

PS	opt;

int podphoto(), quipu_pe_cmp();
void CreateWidgets(), Startup(), XdiLoop(), displayStartupError();
char *cnnct_bind();
void kill_message(), message();

void user_tailor();
void read_args(), quit();

main (argc, argv)
unsigned int argc;
char **argv;
{
  char *mess;

  print_parse_errors = FALSE;
  quipu_syntaxes();
  want_oc_hierarchy();

  namestr[0] = '\0';
  passwd[0] = '\0';

  if ((opt = ps_alloc (std_open)) == NULLPS)
     fatal (-62, "ps_alloc failed");
  if (std_setup (opt, stderr) == NOTOK)
     fatal (-63, "std_setup failed");

  toplevel = XtInitialize("X-Directory", "Xdi", NULL, 0,
                               &argc, argv);

  read_args(&argc, &argv);
  dsap_init((int *) 0, &argv);

  user_tailor();

  if(local_dit && *local_dit)
      (void) strcpy(base_path, local_dit);

  make_friendly(friendly_base_path, base_path);

  CreateWidgets();
  Startup();
  message("Connecting to Directory........"); 
  if ((mess = cnnct_bind()) != NULLCP) {
    kill_message();
    displayStartupError(mess);
    XtMainLoop();
    quit(1);
  } 

  set_attribute_syntax (str2syntax("photo"),
			(IFP)pe_cpy,    NULLIFP,
			NULLIFP,        podphoto,
			(IFP)pe_cpy,    quipu_pe_cmp,
			pe_free,        NULLCP,
			NULLIFP,        TRUE );

  kill_message();
  XdiLoop();

  exit (0);
}

void read_args(acptr, avptr)
unsigned int *acptr;
char ***avptr;
{
  register char *cp;
  char **av;
  int count;

  if (acptr == (unsigned int *) NULL) return;
  if (*acptr <= 1) return;

  av = *avptr;
  av++, count = 1;

  while ((cp = *av) && (*cp == '-')) {
    switch (*++cp) {
    case 'u':
      if (*++av != NULLCP) (void) strcpy(namestr, *av);
      count++;
      break;
    case 'p':
      if (lexequ(*av, "-partial") == 0) FullList = 0;
      else if ((lexequ(*av, "-password") == 0) && (*++av != NULLCP)) 
	   (void) strcpy(passwd, *av);
      count++;
      break;
    case 'T':
      if (*++av != NULLCP) oidtable = *av;
      count++;
      break;
    case 'c':
      if (*++av != NULLCP) myname = *av;
      count++;
      break;
    case 'a':
      if (lexequ(*av, "-attr") == 0) AttrSrch = 1;
      count++;
      break;
    case 'f':
      if (lexequ(*av, "-full") == 0) FullList = 1;
      count++;
      break;
    case 'm':
      if (lexequ(*av, "-move") == 0) MoveOnSrch = 1;
      count++;
      break;
    case 't':
      if (lexequ(*av, "-test") != 0) {
	if (*++av != NULLCP) tailfile = *av;
	count++;
      } else {
	testing = TRUE;
      }
      break;
    }
    av++;
    count++;
  }
  *acptr -= count;
  *avptr = av;
}

void quit(sig)
int     sig;
{
  (void) ds_unbind();
  exit(sig);
}


advise (va_alist)
va_dcl
{
  int     code;
  va_list ap;
  extern LLog * log_dsap;

  va_start (ap);

  code = va_arg (ap, int);
  (void) _ll_log (log_dsap, code, ap);

  va_end (ap);
}
